@@ -41,31 +41,34 @@
 //! }
 //! ```
 
-use std::ops;
+use std::{ops, str};
 
 pub mod errors;
 use errors::*;
 mod luapat;
 use luapat::*;
 
-
 /// Represents a Lua string pattern and the results of a match
 pub struct LuaPattern<'a> {
     patt: &'a [u8],
     matches: [LuaMatch; LUA_MAXCAPTURES],
-    n_match: usize
+    n_match: usize,
 }
 
-impl <'a> LuaPattern<'a> {
+impl<'a> LuaPattern<'a> {
     /// Maybe create a new Lua pattern from a slice of bytes
-    pub fn from_bytes_try (bytes: &'a [u8]) -> Result<LuaPattern<'a>,PatternError> {
+    pub fn from_bytes_try(bytes: &'a [u8]) -> Result<LuaPattern<'a>, PatternError> {
         str_check(bytes)?;
-        let matches = [LuaMatch{start: 0, end: 0}; LUA_MAXCAPTURES];
-        Ok(LuaPattern{patt: bytes, matches: matches, n_match: 0})
+        let matches = [LuaMatch { start: 0, end: 0 }; LUA_MAXCAPTURES];
+        Ok(LuaPattern {
+            patt: bytes,
+            matches: matches,
+            n_match: 0,
+        })
     }
 
     /// Maybe create a new Lua pattern from a string
-    pub fn new_try(patt: &'a str) -> Result<LuaPattern<'a>,PatternError> {
+    pub fn new_try(patt: &'a str) -> Result<LuaPattern<'a>, PatternError> {
         LuaPattern::from_bytes_try(patt.as_bytes())
     }
 
@@ -75,7 +78,7 @@ impl <'a> LuaPattern<'a> {
     }
 
     /// Create a new Lua pattern from a slice of bytes, panicking if bad
-    pub fn from_bytes (bytes: &'a [u8]) -> LuaPattern<'a> {
+    pub fn from_bytes(bytes: &'a [u8]) -> LuaPattern<'a> {
         LuaPattern::from_bytes_try(bytes).expect("bad pattern")
     }
 
@@ -89,7 +92,8 @@ impl <'a> LuaPattern<'a> {
     /// assert_eq!(&bytes[m.range()], &[0xFE,0xEE,0xEE,0xED]);
     /// ```
     pub fn matches_bytes(&mut self, s: &[u8]) -> bool {
-        self.n_match = str_match(s,self.patt,&mut self.matches).expect("Should not fail - report as bug");
+        self.n_match =
+            str_match(s, self.patt, &mut self.matches).expect("Should not fail - report as bug");
         self.n_match > 0
     }
 
@@ -127,11 +131,13 @@ impl <'a> LuaPattern<'a> {
     /// assert_eq!(int,"233");
     /// assert_eq!(rest,"hello");
     /// ```
-    pub fn match_maybe_2<'t>(&mut self, text: &'t str) -> Option<(&'t str,&'t str)> {
+    pub fn match_maybe_2<'t>(&mut self, text: &'t str) -> Option<(&'t str, &'t str)> {
         if self.matches(text) {
             let cc = self.match_captures(text);
-            if cc.num_matches() != 3 { return None; }
-            Some((cc.get(1),cc.get(2)))
+            if cc.num_matches() != 3 {
+                return None;
+            }
+            Some((cc.get(1), cc.get(2)))
         } else {
             None
         }
@@ -146,11 +152,38 @@ impl <'a> LuaPattern<'a> {
     /// assert_eq!(m,"11");
     /// assert_eq!(d,"10");
     /// ```
-    pub fn match_maybe_3<'t>(&mut self, text: &'t str) -> Option<(&'t str,&'t str,&'t str)> {
+    pub fn match_maybe_3<'t>(&mut self, text: &'t str) -> Option<(&'t str, &'t str, &'t str)> {
         if self.matches(text) {
             let cc = self.match_captures(text);
-            if cc.num_matches() != 4 { return None; }
-            Some((cc.get(1),cc.get(2),cc.get(3)))
+            if cc.num_matches() != 4 {
+                return None;
+            }
+            Some((cc.get(1), cc.get(2), cc.get(3)))
+        } else {
+            None
+        }
+    }
+
+    /// Match a string, returning first four explicit captures if successful
+    ///
+    /// ```
+    /// let mut p = lua_patterns::LuaPattern::new("(%d+)/(%d+)/(%d+):(%S+)");
+    /// let (y,m,d,r) = p.match_maybe_4("2017/11/10:rest").unwrap();
+    /// assert_eq!(y,"2017");
+    /// assert_eq!(m,"11");
+    /// assert_eq!(d,"10");
+    /// assert_eq!(r,"rest");
+    /// ```
+    pub fn match_maybe_4<'t>(
+        &mut self,
+        text: &'t str,
+    ) -> Option<(&'t str, &'t str, &'t str, &'t str)> {
+        if self.matches(text) {
+            let cc = self.match_captures(text);
+            if cc.num_matches() != 5 {
+                return None;
+            }
+            Some((cc.get(1), cc.get(2), cc.get(3), cc.get(4)))
         } else {
             None
         }
@@ -179,8 +212,18 @@ impl <'a> LuaPattern<'a> {
     ///     assert_eq!(cc.get(1), "hello");
     /// }
     /// ```
-    pub fn match_captures<'b,'c>(&'c self, text: &'b str) -> Captures<'a,'b,'c> {
-        Captures {m: self, text: text}
+    /// The result is also an iterator over the captures:
+    /// ```rust
+    /// let text = "  hello one";
+    /// let mut m = lua_patterns::LuaPattern::new("(%S+) one");
+    /// if m.matches(text) {
+    ///     let mut iter = m.match_captures(text);
+    ///     assert_eq!(iter.next(), Some("hello one"));
+    ///     assert_eq!(iter.next(), Some("hello"));
+    /// }
+    /// ```    
+    pub fn match_captures<'b, 'c>(&'c self, text: &'b str) -> Captures<'a, 'b, 'c> {
+        Captures::new(self, text)
     }
 
     /// Match and collect all captures into the provided vector.
@@ -217,9 +260,9 @@ impl <'a> LuaPattern<'a> {
     /// assert_eq!(m.capture(1),1..6);
     /// ```
     pub fn capture(&self, i: usize) -> ops::Range<usize> {
-        ops::Range{
+        ops::Range {
             start: self.matches[i].start as usize,
-            end: self.matches[i].end as usize
+            end: self.matches[i].end as usize,
         }
     }
 
@@ -228,7 +271,7 @@ impl <'a> LuaPattern<'a> {
     /// If there are no matches, this is the same as `range`,
     /// otherwise it's `capture(1)`
     pub fn first_capture(&self) -> ops::Range<usize> {
-        let idx = if self.n_match > 1 {1} else {0};
+        let idx = if self.n_match > 1 { 1 } else { 0 };
         self.capture(idx)
     }
 
@@ -243,8 +286,11 @@ impl <'a> LuaPattern<'a> {
     /// let split: Vec<_> = m.gmatch("dog  cat leopard wolf").collect();
     /// assert_eq!(split,&["dog","cat","leopard","wolf"]);
     /// ```
-    pub fn gmatch<'b,'c>(&'c mut self, text: &'b str) -> GMatch<'a,'b,'c> {
-        GMatch{m: self, text: text}
+    pub fn gmatch<'b, 'c>(&'c mut self, text: &'b str) -> GMatch<'a, 'b, 'c> {
+        GMatch {
+            m: self,
+            text: text,
+        }
     }
 
     /// An iterator over all captures in a string.
@@ -259,8 +305,11 @@ impl <'a> LuaPattern<'a> {
     ///       .map(|cc| cc.get(1)).collect();
     /// assert_eq!(split,&["d","c","l","w"]);
     /// ```
-    pub fn gmatch_captures<'b,'c>(&'c mut self, text: &'b str) -> GMatchCaptures<'a,'b,'c> {
-        GMatchCaptures{m: self, text: text}
+    pub fn gmatch_captures<'b, 'c>(&'c mut self, text: &'b str) -> GMatchCaptures<'a, 'b, 'c> {
+        GMatchCaptures {
+            m: self,
+            text: text,
+        }
     }
 
     /// An iterator over all matches in a slice of bytes.
@@ -274,8 +323,11 @@ impl <'a> LuaPattern<'a> {
     /// assert_eq!(iter.next().unwrap(), &[0x01,0x01,0x01]);
     /// assert_eq!(iter.next(), None);
     /// ```
-    pub fn gmatch_bytes<'b>(&'a mut self, bytes: &'b [u8]) -> GMatchBytes<'a,'b> {
-        GMatchBytes{m: self, bytes: bytes}
+    pub fn gmatch_bytes<'b>(&'a mut self, bytes: &'b [u8]) -> GMatchBytes<'a, 'b> {
+        GMatchBytes {
+            m: self,
+            bytes: bytes,
+        }
     }
 
     /// Globally substitute all matches with a replacement
@@ -288,8 +340,10 @@ impl <'a> LuaPattern<'a> {
     /// );
     /// assert_eq!(res, "hello DOLLY you're so FINE!");
     /// ```
-    pub fn gsub_with <F> (&mut self, text: &str, lookup: F) -> String
-    where F: Fn(Captures)-> String {
+    pub fn gsub_with<F>(&mut self, text: &str, lookup: F) -> String
+    where
+        F: Fn(Captures) -> String,
+    {
         let mut slice = text;
         let mut res = String::new();
         while self.matches(slice) {
@@ -297,7 +351,7 @@ impl <'a> LuaPattern<'a> {
             let all = self.range();
             // append everything up to match
             res.push_str(&slice[0..all.start]);
-            let captures = Captures{m: self, text: slice};
+            let captures = Captures::new(self, slice);
             let repl = lookup(captures);
             res.push_str(&repl);
             slice = &slice[all.end..];
@@ -316,23 +370,50 @@ impl <'a> LuaPattern<'a> {
     /// let res = m.gsub("a=2; b=3; c = 4;", "'%2':%1 ");
     /// assert_eq!(res,"'2':a '3':b '4':c ");
     /// ```
-    pub fn gsub (&mut self, text: &str, repl: &str) -> String {
-        let repl = generate_gsub_patterns(repl);
+    pub fn gsub(&mut self, text: &str, repl: &str) -> String {
+        String::from_utf8(self.gsub_bytes(text.as_bytes(), repl.as_bytes())).unwrap()
+    }
+
+    /// Globally substitute all matches with a replacement string
+    ///
+    /// There will be an error if the result is bad UTF-8 (consider using `gsub_bytes`)
+    pub fn gsub_checked(&mut self, text: &str, repl: &str) -> Result<String, PatternError> {
+        String::from_utf8(self.gsub_bytes(text.as_bytes(), repl.as_bytes()))
+            .map_err(|e| PatternError::Utf8(e))
+    }
+
+    /// Globally substitute all _byte_ matches with replacement bytes
+    ///
+    /// Like `gsub` the replacement bytes may contain b"%0" etc
+    ///
+    /// ```
+    /// let bytes = &[0xAA,0x01,0x02,0x03,0xBB];
+    /// let patt = &[0x01,0x02];
+    /// let mut m = lua_patterns::LuaPattern::from_bytes(patt);
+    /// let res = m.gsub_bytes(bytes,&[0xFF]);
+    /// assert_eq!(res, &[0xAA,0xFF,0x03,0xBB]);
+    /// ```
+    pub fn gsub_bytes(&mut self, text: &[u8], repl: &[u8]) -> Vec<u8> {
+        let repl = ByteSubst::gsub_patterns(repl);
         let mut slice = text;
-        let mut res = String::new();
-        while self.matches(slice) {
+        let mut res = Vec::new();
+        while self.matches_bytes(slice) {
             let all = self.range();
-            res.push_str(&slice[0..all.start]);
-            let captures = Captures{m: self, text: slice};
+            let capture = &slice[0..all.start];
+            res.extend_from_slice(capture);
+            let captures = ByteCaptures {
+                m: self,
+                bytes: slice,
+            };
             for r in &repl {
-                match *r {
-                    Subst::Text(ref s) => res.push_str(&s),
-                    Subst::Capture(i) => res.push_str(captures.get(i))
+                match r {
+                    ByteSubst::Bytes(s) => res.extend_from_slice(s),
+                    ByteSubst::Capture(i) => res.extend_from_slice(captures.get(*i)),
                 }
             }
             slice = &slice[all.end..];
         }
-        res.push_str(slice);
+        res.extend_from_slice(slice);
         res
     }
 
@@ -346,15 +427,20 @@ impl <'a> LuaPattern<'a> {
     /// let res = m.gsub_bytes_with(bytes,|cc| vec![0xFF]);
     /// assert_eq!(res, &[0xAA,0xFF,0x03,0xBB]);
     /// ```
-    pub fn gsub_bytes_with <F> (&mut self, bytes: &[u8], lookup: F) -> Vec<u8>
-    where F: Fn(ByteCaptures)-> Vec<u8> {
+    pub fn gsub_bytes_with<F>(&mut self, bytes: &[u8], lookup: F) -> Vec<u8>
+    where
+        F: Fn(ByteCaptures) -> Vec<u8>,
+    {
         let mut slice = bytes;
         let mut res = Vec::new();
         while self.matches_bytes(slice) {
             let all = self.range();
             let capture = &slice[0..all.start];
             res.extend_from_slice(capture);
-            let captures = ByteCaptures{m: self, bytes: slice};
+            let captures = ByteCaptures {
+                m: self,
+                bytes: slice,
+            };
             let repl = lookup(captures);
             res.extend(repl);
             slice = &slice[all.end..];
@@ -362,82 +448,68 @@ impl <'a> LuaPattern<'a> {
         res.extend_from_slice(slice);
         res
     }
-
 }
 
 #[derive(Debug)]
-pub enum Subst {
-    Text(String),
-    Capture(usize)
+pub enum ByteSubst {
+    Bytes(Vec<u8>),
+    Capture(usize),
 }
 
-impl Subst {
-    fn new_text(text: &str) -> Subst {
-        Subst::Text(text.to_string())
-    }
-}
-
-pub fn generate_gsub_patterns(repl: &str) -> Vec<Subst> {
-    let mut m = LuaPattern::new("%%([%%%d])");
-    let mut res = Vec::new();
-    let mut slice = repl;
-    while m.matches(slice) {
-        let all = m.range();
-        let before = &slice[0..all.start];
-        if before != "" {
-            res.push(Subst::new_text(before));
-        }
-        let capture = &slice[m.capture(1)];
-        if capture == "%" { // escaped literal '%'
-            res.push(Subst::new_text("%"));
-        } else { // has to be a digit
-            let index: usize = capture.parse().unwrap();
-            res.push(Subst::Capture(index));
-        }
-        slice = &slice[all.end..];
-    }
-    res.push(Subst::new_text(slice));
-    res
-}
-
-pub struct Substitute {
-    repl: Vec<Subst>
-}
-
-impl Substitute {
-    pub fn new(repl: &str) -> Substitute {
-        Substitute{
-            repl: generate_gsub_patterns(repl)
-        }
+impl ByteSubst {
+    fn new_bytes(bytes: &[u8]) -> Self {
+        Self::Bytes(bytes.to_vec())
     }
 
-    pub fn subst(&self, patt: &LuaPattern, text: &str) -> String {
-        let mut res = String::new();
-        let captures = patt.match_captures(text);
-        for r in &self.repl {
-            match *r {
-                Subst::Text(ref s) => res.push_str(&s),
-                Subst::Capture(i) => res.push_str(captures.get(i))
+    pub fn gsub_patterns(repl: &[u8]) -> Vec<Self> {
+        let mut m = LuaPattern::new("%%([%%%d])");
+        let mut res = Vec::new();
+        let mut slice = repl;
+        while m.matches_bytes(slice) {
+            let all = m.range();
+            let before = &slice[0..all.start];
+            if before != b"" {
+                res.push(Self::new_bytes(before));
             }
+            let capture = &slice[m.capture(1)];
+            if capture == b"%" {
+                // escaped literal '%'
+                res.push(Self::new_bytes(b"%"));
+            } else {
+                // has to be a digit
+                let index: usize = str::from_utf8(capture).unwrap().parse().unwrap();
+                res.push(Self::Capture(index));
+            }
+            slice = &slice[all.end..];
         }
+        res.push(Self::new_bytes(slice));
         res
     }
-
 }
 
-
-
 /// Low-overhead convenient access to string match captures
 // note: there are three borrows going on here.
 // The lifetime 'a is for the _pattern_, the lifetime 'b is
 // for the _source string_, and 'c is for the reference to LuaPattern
 // And the LuaPattern reference cannot live longer than the pattern reference
-pub struct Captures<'a,'b,'c> where 'a: 'c {
+pub struct Captures<'a, 'b, 'c>
+where
+    'a: 'c,
+{
     m: &'c LuaPattern<'a>,
-    text: &'b str
+    text: &'b str,
+    i_match: usize,
 }
 
-impl <'a,'b,'c> Captures<'a,'b,'c> {
+impl<'a, 'b, 'c> Captures<'a, 'b, 'c> {
+    fn new(m: &'c LuaPattern<'a>, text: &'b str) -> Self {
+        Self {
+            i_match: 0,
+            m,
+            text,
+        }
+    }
+
     /// get the capture as a string slice
     pub fn get(&self, i: usize) -> &'b str {
         &self.text[self.m.capture(i)]
@@ -449,14 +521,27 @@ impl <'a,'b,'c> Captures<'a,'b,'c> {
     }
 }
 
+impl<'a, 'b, 'c> Iterator for Captures<'a, 'b, 'c> {
+    type Item = &'b str;
 
-/// Low-overhead convenient access to byte match captures
-pub struct ByteCaptures<'a,'b> {
-    m: &'a LuaPattern<'a>,
-    bytes: &'b [u8]
+    fn next(&mut self) -> Option<Self::Item> {
+        if self.i_match == self.num_matches() {
+            None
+        } else {
+            let res = Some(self.get(self.i_match));
+            self.i_match += 1;
+            res
+        }
+    }
 }
 
-impl <'a,'b> ByteCaptures<'a,'b> {
+/// Low-overhead convenient access to byte match captures
+pub struct ByteCaptures<'a, 'b> {
+    m: &'a LuaPattern<'a>,
+    bytes: &'b [u8],
+}
+
+impl<'a, 'b> ByteCaptures<'a, 'b> {
     /// get the capture as a byte slice
     pub fn get(&self, i: usize) -> &'b [u8] {
         &self.bytes[self.m.capture(i)]
@@ -470,16 +555,19 @@ impl <'a,'b> ByteCaptures<'a,'b> {
 
 /// Iterator for all string slices from `gmatch`
 // note lifetimes as for Captures above!
-pub struct GMatch<'a,'b,'c> where 'a: 'c {
+pub struct GMatch<'a, 'b, 'c>
+where
+    'a: 'c,
+{
     m: &'c mut LuaPattern<'a>,
-    text: &'b str
+    text: &'b str,
 }
 
-impl <'a,'b,'c>Iterator for GMatch<'a,'b,'c> {
+impl<'a, 'b, 'c> Iterator for GMatch<'a, 'b, 'c> {
     type Item = &'b str;
 
     fn next(&mut self) -> Option<Self::Item> {
-        if ! self.m.matches(self.text) {
+        if !self.m.matches(self.text) {
             None
         } else {
             let slice = &self.text[self.m.first_capture()];
@@ -487,27 +575,25 @@ impl <'a,'b,'c>Iterator for GMatch<'a,'b,'c> {
             Some(slice)
         }
     }
-
 }
 
 /// Unsafe version of Captures, needed for gmatch_captures
 // It's unsafe because the lifetime only depends on the original
 // text, not the borrowed matches.
-pub struct CapturesUnsafe<'b>{
+pub struct CapturesUnsafe<'b> {
     matches: *const LuaMatch,
-    text: &'b str
+    text: &'b str,
 }
 
-impl <'b> CapturesUnsafe<'b> {
+impl<'b> CapturesUnsafe<'b> {
     /// get the capture as a string slice
     pub fn get(&self, i: usize) -> &'b str {
         unsafe {
             let p = self.matches.offset(i as isize);
-            let range =
-                ops::Range{
-                    start: (*p).start as usize,
-                    end: (*p).end as usize
-                };
+            let range = ops::Range {
+                start: (*p).start as usize,
+                end: (*p).end as usize,
+            };
             &self.text[range]
         }
     }
@@ -516,38 +602,46 @@ impl <'b> CapturesUnsafe<'b> {
 /// Streaming iterator for all captures from `gmatch_captures`
 // lifetimes as for Captures above!
 // 'a is pattern, 'b is text, 'c is ref to LuaPattern
-pub struct GMatchCaptures<'a,'b,'c> where 'a: 'c {
+pub struct GMatchCaptures<'a, 'b, 'c>
+where
+    'a: 'c,
+{
     m: &'c mut LuaPattern<'a>,
-    text: &'b str
+    text: &'b str,
 }
 
-impl <'a,'b,'c> Iterator for GMatchCaptures<'a,'b,'c>  where 'a: 'c {
+impl<'a, 'b, 'c> Iterator for GMatchCaptures<'a, 'b, 'c>
+where
+    'a: 'c,
+{
     type Item = CapturesUnsafe<'b>;
 
     fn next(&mut self) -> Option<Self::Item> {
-        if ! self.m.matches(self.text) {
+        if !self.m.matches(self.text) {
             None
         } else {
             let split = self.text.split_at(self.m.range().end);
             self.text = split.1;
             let match_ptr: *const LuaMatch = self.m.matches.as_ptr();
-            Some(CapturesUnsafe{matches: match_ptr, text: split.0})
+            Some(CapturesUnsafe {
+                matches: match_ptr,
+                text: split.0,
+            })
         }
     }
-
 }
 
 /// Iterator for all byte slices from `gmatch_bytes`
-pub struct GMatchBytes<'a,'b> {
+pub struct GMatchBytes<'a, 'b> {
     m: &'a mut LuaPattern<'a>,
-    bytes: &'b [u8]
+    bytes: &'b [u8],
 }
 
-impl <'a,'b>Iterator for GMatchBytes<'a,'b> {
+impl<'a, 'b> Iterator for GMatchBytes<'a, 'b> {
     type Item = &'b [u8];
 
     fn next(&mut self) -> Option<Self::Item> {
-        if ! self.m.matches_bytes(self.bytes) {
+        if !self.m.matches_bytes(self.bytes) {
             None
         } else {
             let slice = &self.bytes[self.m.first_capture()];
@@ -555,18 +649,17 @@ impl <'a,'b>Iterator for GMatchBytes<'a,'b> {
             Some(slice)
         }
     }
-
 }
 
 /// Build a byte Lua pattern, optionally escaping 'magic' characters
 pub struct LuaPatternBuilder {
-    bytes: Vec<u8>
+    bytes: Vec<u8>,
 }
 
 impl LuaPatternBuilder {
     /// Create a new Lua pattern builder
     pub fn new() -> LuaPatternBuilder {
-        LuaPatternBuilder{bytes: Vec::new()}
+        LuaPatternBuilder { bytes: Vec::new() }
     }
 
     /// Add unescaped characters from a string
@@ -620,7 +713,7 @@ impl LuaPatternBuilder {
     /// ```
     pub fn bytes(&mut self, b: &[u8]) -> &mut Self {
         let mut m = LuaPattern::new("[%-%.%+%[%]%(%)%$%^%%%?%*]");
-        let bb = m.gsub_bytes_with(b,|cc| {
+        let bb = m.gsub_bytes_with(b, |cc| {
             let mut res = Vec::new();
             res.push(b'%');
             res.push(cc.get(0)[0]);
@@ -661,7 +754,9 @@ impl LuaPatternBuilder {
     /// ```
     pub fn hex_to_bytes(s: &str) -> Vec<u8> {
         let mut m = LuaPattern::new("%x%x");
-        m.gmatch(s).map(|pair| u8::from_str_radix(pair,16).unwrap()).collect()
+        m.gmatch(s)
+            .map(|pair| u8::from_str_radix(pair, 16).unwrap())
+            .collect()
     }
 
     /// Utility to create a hex string from a slice of bytes
@@ -672,9 +767,8 @@ impl LuaPatternBuilder {
     ///
     /// ```
     pub fn bytes_to_hex(s: &[u8]) -> String {
-        s.iter().map(|b| format!("{:02X}",b)).collect()
+        s.iter().map(|b| format!("{:02X}", b)).collect()
     }
-
 }
 
 #[cfg(test)]
@@ -684,7 +778,7 @@ mod tests {
     #[test]
     fn captures_and_matching() {
         let mut m = LuaPattern::new("(one).+");
-        assert_eq!(m.captures(" one two"), &["one two","one"]);
+        assert_eq!(m.captures(" one two"), &["one two", "one"]);
         let empty: &[&str] = &[];
         assert_eq!(m.captures("four"), empty);
 
@@ -711,15 +805,14 @@ mod tests {
         assert_eq!(cc[0], "hello= bonzo dog");
         assert_eq!(cc[1], "hello");
         assert_eq!(cc[2], "bonzo dog");
-
     }
 
     #[test]
     fn multiple_captures() {
         let mut p = LuaPattern::new("%s*(%d+)%s+(%S+)");
-        let (int,rest) = p.match_maybe_2(" 233   hello dolly").unwrap();
-        assert_eq!(int,"233");
-        assert_eq!(rest,"hello");
+        let (int, rest) = p.match_maybe_2(" 233   hello dolly").unwrap();
+        assert_eq!(int, "233");
+        assert_eq!(rest, "hello");
     }
 
     #[test]
@@ -750,9 +843,9 @@ mod tests {
         use std::collections::HashMap;
 
         let mut m = LuaPattern::new("%$(%S+)");
-        let res = m.gsub_with("hello $dolly you're so $fine!",
-            |cc| cc.get(1).to_uppercase()
-        );
+        let res = m.gsub_with("hello $dolly you're so $fine!", |cc| {
+            cc.get(1).to_uppercase()
+        });
         assert_eq!(res, "hello DOLLY you're so FINE!");
 
         let mut map = HashMap::new();
@@ -761,34 +854,35 @@ mod tests {
         map.insert("good-looking", "pretty");
 
         let mut m = LuaPattern::new("%$%((.-)%)");
-        let res = m.gsub_with("hello $(dolly) you're so $(fine) and $(good-looking)",
-            |cc| map.get(cc.get(1)).unwrap_or(&"?").to_string()
+        let res = m.gsub_with(
+            "hello $(dolly) you're so $(fine) and $(good-looking)",
+            |cc| map.get(cc.get(1)).unwrap_or(&"?").to_string(),
         );
         assert_eq!(res, "hello baby you're so cool and pretty");
 
         let mut m = LuaPattern::new("%s+");
-        let res = m.gsub("hello dolly you're so fine","");
+        let res = m.gsub("hello dolly you're so fine", "");
         assert_eq!(res, "hellodollyyou'resofine");
 
         let mut m = LuaPattern::new("(%S+)%s*=%s*(%S+);%s*");
         let res = m.gsub("a=2; b=3; c = 4;", "'%2':%1 ");
-        assert_eq!(res,"'2':a '3':b '4':c ");
+        assert_eq!(res, "'2':a '3':b '4':c ");
     }
 
     #[test]
     fn bad_patterns() {
-       let bad = [
-        ("bonzo %","malformed pattern (ends with '%')"),
-        ("bonzo (dog%(","unfinished capture"),
-        ("alles [%a%[","malformed pattern (missing ']')"),
-        ("bonzo (dog (cat)","unfinished capture"),
-        ("frodo %f[%A","malformed pattern (missing ']')"),
-        ("frodo (1) (2(3)%2)%1","invalid capture index %2"),
+        let bad = [
+            ("bonzo %", "malformed pattern (ends with '%')"),
+            ("bonzo (dog%(", "unfinished capture"),
+            ("alles [%a%[", "malformed pattern (missing ']')"),
+            ("bonzo (dog (cat)", "unfinished capture"),
+            ("frodo %f[%A", "malformed pattern (missing ']')"),
+            ("frodo (1) (2(3)%2)%1", "invalid capture index %2"),
         ];
         for p in bad.iter() {
             let res = LuaPattern::new_try(p.0);
             if let Err(e) = res {
-                assert_eq!(e, PatternError(p.1.into()));
+                assert_eq!(e, PatternError::Pattern(p.1.into()));
             } else {
                 panic!("false positive");
             }
